@@ -1,6 +1,6 @@
 ;;; exwm-core.el --- Core definitions  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -99,12 +99,11 @@ nil: Disable timestamp."
 (declare-function exwm-workspace-switch "exwm-workspace.el"
                   (frame-or-index &optional force))
 
-(defvaralias 'exwm-debug 'exwm-debug-mode) ;; Not obsolete
+(defvaralias 'exwm-debug 'exwm-debug-mode)
 (define-minor-mode exwm-debug-mode
   "Debug-logging enabled if non-nil."
   :global t
   :group 'exwm-debug)
-(define-obsolete-function-alias 'exwm-debug #'exwm-debug-mode "0.30")
 
 (defmacro exwm--debug (&rest forms)
   "Evaluate FORMS if `exwm-debug-mode' is active."
@@ -284,41 +283,38 @@ One of `line-mode' or `char-mode'.")
 ;; _MOTIF_WM_HINTS
 (defvar-local exwm--mwm-hints-decorations t)
 
-(defvar exwm-mode-map
-  (let ((map (make-sparse-keymap)))
-    (define-key map "\C-c\C-d\C-l" #'xcb-debug:clear)
-    (define-key map "\C-c\C-d\C-m" #'xcb-debug:mark)
-    (define-key map "\C-c\C-d\C-t" #'exwm-debug-mode)
-    (define-key map "\C-c\C-f" #'exwm-layout-set-fullscreen)
-    (define-key map "\C-c\C-h" #'exwm-floating-hide)
-    (define-key map "\C-c\C-k" #'exwm-input-release-keyboard)
-    (define-key map "\C-c\C-m" #'exwm-workspace-move-window)
-    (define-key map "\C-c\C-q" #'exwm-input-send-next-key)
-    (define-key map "\C-c\C-t\C-f" #'exwm-floating-toggle-floating)
-    (define-key map "\C-c\C-t\C-m" #'exwm-layout-toggle-mode-line)
-    map)
-  "Keymap for `exwm-mode'.")
+(defvar-keymap exwm-mode-map
+  :doc "Keymap for `exwm-mode'."
+  "C-c C-d C-l" #'xcb-debug:clear
+  "C-c C-d C-m" #'xcb-debug:mark
+  "C-c C-d C-t" #'exwm-debug-mode
+  "C-c C-f" #'exwm-layout-set-fullscreen
+  "C-c C-h" #'exwm-floating-hide
+  "C-c C-k" #'exwm-input-release-keyboard
+  "C-c C-m" #'exwm-workspace-move-window
+  "C-c C-q" #'exwm-input-send-next-key
+  "C-c C-t C-f" #'exwm-floating-toggle-floating
+  "C-c C-t C-m" #'exwm-layout-toggle-mode-line)
 
-(defvar exwm--kmacro-map
-  (let ((map (make-sparse-keymap)))
-    (define-key map [t]
-      (lambda ()
-        (interactive)
-        (cond
-         ((or exwm-input-line-mode-passthrough
-              ;; Do not test `exwm-input--during-command'.
-              (active-minibuffer-window)
-              (memq last-input-event exwm-input--global-prefix-keys)
-              (memq last-input-event exwm-input-prefix-keys)
-              (lookup-key exwm-mode-map (vector last-input-event))
-              (gethash last-input-event exwm-input--simulation-keys))
-          (set-transient-map (make-composed-keymap (list exwm-mode-map
-                                                         global-map)))
-          (push last-input-event unread-command-events))
-         (t
-          (exwm-input--fake-key last-input-event)))))
-    map)
-  "Keymap used when executing keyboard macros.")
+(defun exwm--kmacro-self-insert-command ()
+  "The EXWM kmacro equivalent of `self-insert-command'."
+  (interactive)
+  (cond
+   ((or exwm-input-line-mode-passthrough
+        (active-minibuffer-window)
+        (memq last-input-event exwm-input--global-prefix-keys)
+        (memq last-input-event exwm-input-prefix-keys)
+        (lookup-key exwm-mode-map (vector last-input-event))
+        (gethash last-input-event exwm-input--simulation-keys))
+    (set-transient-map (make-composed-keymap (list exwm-mode-map global-map)))
+    (push last-input-event unread-command-events))
+   (t
+    (exwm-input--fake-key last-input-event))))
+(put 'exwm--kmacro-self-insert-command 'completion-predicate #'ignore)
+
+(defvar-keymap exwm--kmacro-map
+  :doc "Keymap used when executing keyboard macros."
+  "<t>" #'exwm--kmacro-self-insert-command)
 
 ;; This menu mainly acts as an reminder for users.  Thus it should be as
 ;; detailed as possible, even some entries do not make much sense here.
@@ -326,30 +322,22 @@ One of `line-mode' or `char-mode'.")
 (easy-menu-define exwm-mode-menu exwm-mode-map
   "Menu for `exwm-mode'."
   `("EXWM"
-    "---"
-    "*General*"
-    "---"
-    ["Toggle floating" exwm-floating-toggle-floating]
-    ["Toggle fullscreen mode" exwm-layout-toggle-fullscreen]
-    ["Hide window" exwm-floating-hide exwm--floating-frame]
-    ["Close window" (kill-buffer (current-buffer))]
-
-    "---"
-    "*Resizing*"
-    "---"
-    ["Toggle mode-line" exwm-layout-toggle-mode-line]
-    ["Enlarge window vertically" exwm-layout-enlarge-window]
-    ["Enlarge window horizontally" exwm-layout-enlarge-window-horizontally]
-    ["Shrink window vertically" exwm-layout-shrink-window]
-    ["Shrink window horizontally" exwm-layout-shrink-window-horizontally]
-
-    "---"
-    "*Keyboard*"
-    "---"
-    ["Toggle keyboard mode" exwm-input-toggle-keyboard]
-    ["Send key" exwm-input-send-next-key (eq exwm--input-mode 'line-mode)]
+    ("General"
+     ["Toggle floating" exwm-floating-toggle-floating]
+     ["Toggle fullscreen mode" exwm-layout-toggle-fullscreen]
+     ["Hide window" exwm-floating-hide exwm--floating-frame]
+     ["Close window" (kill-buffer (current-buffer))])
+    ("Resizing"
+     ["Toggle mode-line" exwm-layout-toggle-mode-line]
+     ["Enlarge window vertically" exwm-layout-enlarge-window]
+     ["Enlarge window horizontally" exwm-layout-enlarge-window-horizontally]
+     ["Shrink window vertically" exwm-layout-shrink-window]
+     ["Shrink window horizontally" exwm-layout-shrink-window-horizontally])
+    ("Keyboard"
+     ["Toggle keyboard mode" exwm-input-toggle-keyboard]
+     ["Send key" exwm-input-send-next-key (eq exwm--input-mode 'line-mode)]
     ;; This is merely a reference.
-    ("Send simulation key" :filter
+     ("Send simulation key" :filter
      ,(lambda (&rest _args)
         (let (result)
           (maphash
@@ -365,29 +353,27 @@ One of `line-mode' or `char-mode'.")
            exwm-input--simulation-keys)
           result)))
 
-    ["Define global binding" exwm-input-set-key]
+     ["Define global binding" exwm-input-set-key])
 
-    "---"
-    "*Workspace*"
-    "---"
-    ["Add workspace" exwm-workspace-add]
-    ["Delete current workspace" exwm-workspace-delete]
-    ["Move workspace to" exwm-workspace-move]
-    ["Swap workspaces" exwm-workspace-swap]
-    ["Move X window to" exwm-workspace-move-window]
-    ["Move X window from" exwm-workspace-switch-to-buffer]
-    ["Toggle minibuffer" exwm-workspace-toggle-minibuffer]
-    ["Switch workspace" exwm-workspace-switch]
-    ;; Place this entry at bottom to avoid selecting others by accident.
-    ("Switch to" :filter
-     ,(lambda (&rest _args)
-        (mapcar (lambda (i)
-                  `[,(format "Workspace %d" i)
-                    ,(lambda ()
-                       (interactive)
-                       (exwm-workspace-switch i))
-                    (/= ,i exwm-workspace-current-index)])
-                (number-sequence 0 (1- (length exwm-workspace--list))))))))
+    ("Workspace"
+     ["Add workspace" exwm-workspace-add]
+     ["Delete current workspace" exwm-workspace-delete]
+     ["Move workspace to" exwm-workspace-move]
+     ["Swap workspaces" exwm-workspace-swap]
+     ["Move X window to" exwm-workspace-move-window]
+     ["Move X window from" exwm-workspace-switch-to-buffer]
+     ["Toggle minibuffer" exwm-workspace-toggle-minibuffer]
+     ["Switch workspace" exwm-workspace-switch]
+     ;; Place this entry at bottom to avoid selecting others by accident.
+     ("Switch to" :active (cdr exwm-workspace--list) :filter
+      ,(lambda (&rest _args)
+         (mapcar (lambda (i)
+                   `[,(format "Workspace %d" i)
+                     ,(lambda ()
+                        (interactive)
+                        (exwm-workspace-switch i))
+                     (/= ,i exwm-workspace-current-index)])
+                 (number-sequence 0 (1- (length exwm-workspace--list)))))))))
 
 (define-derived-mode exwm-mode nil "EXWM"
   "Major mode for managing X windows.
@@ -402,14 +388,19 @@ One of `line-mode' or `char-mode'.")
   ;; Redirect events when executing keyboard macros.
   (push `(executing-kbd-macro . ,exwm--kmacro-map)
         minor-mode-overriding-map-alist)
-  (setq mode-name '(:eval (exwm--mode-name))
-        buffer-read-only t
-        cursor-type nil
-        left-margin-width nil
-        right-margin-width nil
-        left-fringe-width 0
-        right-fringe-width 0
-        vertical-scroll-bar nil))
+  (setq-local mode-name '(:eval (exwm--mode-name))
+              buffer-read-only t
+              cursor-type nil
+              left-margin-width nil
+              right-margin-width nil
+              left-fringe-width 0
+              right-fringe-width 0
+              vertical-scroll-bar nil
+              eldoc-documentation-functions nil
+              mode-line-position nil
+              mode-line-modified nil
+              mode-line-mule-info nil
+              mode-line-remote nil))
 
 (defmacro exwm--global-minor-mode-body (name &optional init exit)
   "Global minor mode body for mode with NAME.
@@ -432,8 +423,5 @@ immediately enabled or disabled."
          (remove-hook 'exwm-exit-hook #',exit)
          (when exwm--connection (,exit)))))))
 
-
-
 (provide 'exwm-core)
-
 ;;; exwm-core.el ends here
