@@ -1,6 +1,6 @@
 ;;; exwm-input.el --- Input Module for EXWM  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -107,9 +107,6 @@ defined in `exwm-mode-map' here."
                                               read-event)
   "Low-level read functions that must be exempted from EXWM input handling.")
 
-(defvar exwm-input--during-command nil
-  "Indicate whether between `pre-command-hook' and `post-command-hook'.")
-
 (defvar exwm-input--global-keys nil "Global key bindings.")
 
 (defvar exwm-input--global-prefix-keys nil
@@ -376,23 +373,25 @@ attempt later."
                                      window)
                 (exwm--defer 0 #'exwm-workspace-switch (selected-frame)))
             ;; The focus is still on the current workspace.
-            (if (not (and (exwm-workspace--minibuffer-own-frame-p)
-                          (minibufferp)))
-                (x-focus-frame (window-frame window))
-              ;; X input focus should be set on the previously selected
-              ;; frame.
-              (x-focus-frame (window-frame (minibuffer-window))))
-            (exwm-input--set-active-window)
+            (let ((frame (if (not (and (exwm-workspace--minibuffer-own-frame-p)
+                                       (minibufferp)))
+                             (window-frame window)
+                           ;; X input focus should be set on the previously
+                           ;; selected frame.
+                           (window-frame (minibuffer-window)))))
+              (x-focus-frame frame)
+              (exwm-input--set-active-window
+               (or (frame-parameter exwm-workspace--current 'exwm-outer-id)
+                   xcb:Window:None)))
             (xcb:flush exwm--connection)))))))
 
-(defun exwm-input--set-active-window (&optional id)
-  "Set _NET_ACTIVE_WINDOW.
-When non-nil, ID is passed as the window data."
+(defun exwm-input--set-active-window (id)
+  "Set _NET_ACTIVE_WINDOW to ID."
   (exwm--log)
   (xcb:+request exwm--connection
       (make-instance 'xcb:ewmh:set-_NET_ACTIVE_WINDOW
                      :window exwm--root
-                     :data (or id xcb:Window:None))))
+                     :data id)))
 
 (defun exwm-input--on-ButtonPress (data _synthetic)
   "Handle ButtonPress event with DATA."
@@ -567,7 +566,6 @@ Notes:
          (when exwm--connection
            (exwm-input--update-global-prefix-keys))))
 
-;;;###autoload
 (defun exwm-input-set-key (key command)
   "Set a global KEY binding to COMMAND.
 
@@ -645,7 +643,6 @@ When non-nil, TEMP-LINE-MODE temporarily puts the window in line mode."
   "Whether EVENT should be passed to Emacs.
 Current buffer must be an `exwm-mode' buffer."
   (or exwm-input-line-mode-passthrough
-      exwm-input--during-command
       ;; Forward the event when there is an incomplete key
       ;; sequence or when the minibuffer is active.
       exwm-input--line-mode-cache
@@ -662,6 +659,7 @@ Current buffer must be an `exwm-mode' buffer."
 (defun exwm-input--noop (&rest _args)
   "A placeholder command."
   (interactive))
+(put #'exwm-input--noop 'completion-predicate #'ignore) ;; Move to declare in Emacs 28
 
 (defun exwm-input--fake-last-command ()
   "Fool some packages into thinking there is a change in the buffer."
@@ -844,7 +842,6 @@ button event."
           (setq exwm--input-mode 'char-mode)
           (run-hooks 'exwm-input-input-mode-change-hook))))))
 
-;;;###autoload
 (defun exwm-input-grab-keyboard (&optional id)
   "Switch to `line-mode`.
 When ID is non-nil, grab key events on its corresponding window."
@@ -856,7 +853,6 @@ When ID is non-nil, grab key events on its corresponding window."
     (exwm-input--grab-keyboard id)
     (exwm-input--update-mode-line id)))
 
-;;;###autoload
 (defun exwm-input-release-keyboard (&optional id)
   "Switch to `char-mode`.
 When ID is non-nil, release keyboard events on its corresponding window."
@@ -868,7 +864,6 @@ When ID is non-nil, release keyboard events on its corresponding window."
     (exwm-input--release-keyboard id)
     (exwm-input--update-mode-line id)))
 
-;;;###autoload
 (defun exwm-input-toggle-keyboard (&optional id)
   "Toggle between `line-mode' and `char-mode'.
 When ID is non-nil, toggle in its correpsonding window."
@@ -912,7 +907,6 @@ When ID is non-nil, toggle in its correpsonding window."
                                    exwm--connection)))))
     (xcb:flush exwm--connection)))
 
-;;;###autoload
 (cl-defun exwm-input-send-next-key (n &optional end-key)
   "Send next N keys to client window.
 N is currently capped at 12.
@@ -1015,12 +1009,6 @@ Notes:
          (set symbol value)
          (exwm-input--set-simulation-keys value)))
 
-(defcustom exwm-input-pre-post-command-blacklist '(exit-minibuffer
-                                                   abort-recursive-edit
-                                                   minibuffer-keyboard-quit)
-  "Commands impossible to detect with `post-command-hook'."
-  :type '(repeat function))
-
 (cl-defun exwm-input--read-keys (prompt stop-key)
   "Read keys with PROMPT until STOP-KEY pressed."
   (let ((cursor-in-echo-area t)
@@ -1034,7 +1022,6 @@ Notes:
     (when (> (length keys) 1)
       (substring keys 0 -1))))
 
-;;;###autoload
 (defun exwm-input-set-simulation-key (original-key simulated-key)
   "Set ORIGINAL-KEY to  SIMULATED-KEY.
 
@@ -1078,7 +1065,6 @@ where both ORIGINAL-KEY and SIMULATED-KEY are key sequences."
   (let ((exwm-input--local-simulation-keys t))
     (exwm-input--set-simulation-keys simulation-keys)))
 
-;;;###autoload
 (cl-defun exwm-input-send-simulation-key (n)
   "Fake N key events according to the last input key sequence."
   (interactive "p")
@@ -1091,7 +1077,6 @@ where both ORIGINAL-KEY and SIMULATED-KEY are key sequences."
       (dolist (key keys)
         (exwm-input--fake-key key)))))
 
-;;;###autoload
 (defmacro exwm-input-invoke-factory (keys)
   "Make a command that invokes KEYS when called.
 
@@ -1106,17 +1091,6 @@ One use is to access the keymap bound to KEYS (as prefix keys) in `char-mode'."
                (exwm-input--unread-event key))
              ',(listify-key-sequence keys)))))
 
-(defun exwm-input--on-pre-command ()
-  "Run in `pre-command-hook'."
-  (unless (or (eq this-command #'exwm-input--noop)
-              (memq this-command exwm-input-pre-post-command-blacklist))
-    (setq exwm-input--during-command t)))
-
-(defun exwm-input--on-post-command ()
-  "Run in `post-command-hook'."
-  (unless (eq this-command #'exwm-input--noop)
-    (setq exwm-input--during-command nil)))
-
 (defun exwm-input--on-minibuffer-setup ()
   "Run in `minibuffer-setup-hook' to grab keyboard if necessary."
   (let* ((window (or (minibuffer-selected-window) ; minibuffer-setup-hook
@@ -1205,9 +1179,6 @@ One use is to access the keymap bound to KEYS (as prefix keys) in `char-mode'."
   (when mouse-autoselect-window
     (xcb:+event exwm--connection 'xcb:EnterNotify
                 #'exwm-input--on-EnterNotify))
-  ;; Control `exwm-input--during-command'
-  (add-hook 'pre-command-hook #'exwm-input--on-pre-command)
-  (add-hook 'post-command-hook #'exwm-input--on-post-command)
   ;; Grab/Release keyboard when minibuffer/echo becomes active/inactive.
   (add-hook 'minibuffer-setup-hook #'exwm-input--on-minibuffer-setup)
   (add-hook 'minibuffer-exit-hook #'exwm-input--on-minibuffer-exit)
@@ -1231,8 +1202,6 @@ One use is to access the keymap bound to KEYS (as prefix keys) in `char-mode'."
   (dolist (fun exwm-input--passthrough-functions)
     (advice-remove fun #'exwm-input--call-with-passthrough))
   (exwm-input--unset-simulation-keys)
-  (remove-hook 'pre-command-hook #'exwm-input--on-pre-command)
-  (remove-hook 'post-command-hook #'exwm-input--on-post-command)
   (remove-hook 'minibuffer-setup-hook #'exwm-input--on-minibuffer-setup)
   (remove-hook 'minibuffer-exit-hook #'exwm-input--on-minibuffer-exit)
   (when exwm-input--echo-area-timer
@@ -1251,8 +1220,5 @@ One use is to access the keymap bound to KEYS (as prefix keys) in `char-mode'."
                        :time xcb:Time:CurrentTime))
     (xcb:flush exwm--connection)))
 
-
-
 (provide 'exwm-input)
-
 ;;; exwm-input.el ends here
