@@ -1,7 +1,7 @@
 ;;; exwm-manage.el --- Window Management Module for  -*- lexical-binding: t -*-
 ;;;                    EXWM
 
-;; Copyright (C) 2015-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -18,7 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -27,6 +27,7 @@
 ;;; Code:
 
 (require 'exwm-core)
+(eval-when-compile (require 'subr-x)) ;; Needed on 28 for when-let*
 
 (defgroup exwm-manage nil
   "Manage."
@@ -250,7 +251,6 @@ This only works when procfs is mounted, which may not be the case on some BSDs."
         (setq default-directory (file-name-as-directory cwd))
       (setq default-directory (expand-file-name "~/")))))
 
-
 (defun exwm-manage--set-client-list ()
   "Set _NET_CLIENT_LIST."
   (exwm--log)
@@ -410,9 +410,9 @@ This only works when procfs is mounted, which may not be the case on some BSDs."
       (if (plist-get exwm--configurations 'char-mode)
           (exwm-input-release-keyboard id)
         (exwm-input-grab-keyboard id))
-      (when-let ((simulation-keys (plist-get exwm--configurations 'simulation-keys)))
+      (when-let* ((simulation-keys (plist-get exwm--configurations 'simulation-keys)))
         (exwm-input-set-local-simulation-keys simulation-keys))
-      (when-let ((prefix-keys (plist-get exwm--configurations 'prefix-keys)))
+      (when-let* ((prefix-keys (plist-get exwm--configurations 'prefix-keys)))
         (setq-local exwm-input-prefix-keys prefix-keys))
       (setq exwm-workspace--switch-history-outdated t)
       (exwm--update-desktop id)
@@ -500,9 +500,7 @@ manager is shutting down."
                (with-current-buffer buffer
                  (let ((kill-buffer-query-functions nil))
                    (kill-buffer buffer))))))
-        (exwm--defer 0 kill-buffer-func buffer)
-        (when (active-minibuffer-window)
-          (exit-minibuffer))))))
+        (exwm--defer 0 kill-buffer-func buffer)))))
 
 (defun exwm-manage--scan ()
   "Search for existing windows and try to manage them."
@@ -826,8 +824,5 @@ SYNTHETIC indicates whether the event is a synthetic event."
   (remove-hook 'delete-frame-functions #'exwm-manage--remove-frame)
   (setq exwm-manage--_MOTIF_WM_HINTS nil))
 
-
-
 (provide 'exwm-manage)
-
 ;;; exwm-manage.el ends here
