@@ -1,7 +1,7 @@
 ;;; exwm-systemtray.el --- System Tray Module for  -*- lexical-binding: t -*-
 ;;;                        EXWM
 
-;; Copyright (C) 2016-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -18,7 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -57,11 +57,6 @@
   :group 'exwm-systemtray
   (exwm--global-minor-mode-body systemtray))
 
-(defun exwm-systemtray-enable ()
-  "Enable EXWM systemtray support."
-  (exwm-systemtray-mode 1))
-(make-obsolete 'exwm-systemtray-enable "Use `exwm-systemtray-mode' instead." "0.30")
-
 (defcustom exwm-systemtray-height nil
   "System tray height.
 
@@ -72,32 +67,43 @@ You shall use the default value if using auto-hide minibuffer."
   "Gap between icons."
   :type 'integer)
 
+(defcustom exwm-systemtray-position 'bottom
+  "Position of the systemtray at the top or bottom.
+The position can be set to an integer.  In this case a
+positive (negative) integer specifies the offset from the top (bottom)
+of the screen."
+  :type '(choice (const top) (const bottom) integer))
+
 (defvar exwm-systemtray--connection nil "The X connection.")
 
 (defvar exwm-systemtray--embedder-window nil "The embedder window.")
 (defvar exwm-systemtray--embedder-window-depth nil
   "The embedder window's depth.")
 
-(defcustom exwm-systemtray-background-color 'workspace-background
+(defcustom exwm-systemtray-background-color 'default
   "Background color of systemtray.
-This should be a color, the symbol `workspace-background' for the background
-color of current workspace frame, or the symbol `transparent' for transparent
-background.
+This should be a color, the symbol `transparent' for transparent
+background, or a face symbol like `default' or `tab-bar'.
 
 Transparent background is not yet supported when Emacs uses 32-bit depth
 visual, as reported by `x-display-planes'.  The X resource \"Emacs.visualClass:
 TrueColor-24\" can be used to force Emacs to use 24-bit depth."
   :type '(choice (const :tag "Transparent" transparent)
-                 (const :tag "Frame background" workspace-background)
+                 (const :tag "Frame background" default)
+                 (const :tag "Tab-bar background" tab-bar)
                  (color :tag "Color"))
   :initialize #'custom-initialize-default
   :set (lambda (symbol value)
+         (when (eq value 'workspace-background)
+           (display-warning 'exwm-systemtray "Use `default' instead\
+ of `workspace-background' for `exwm-systemtray-background-color'.")
+           (setq value 'default))
          (when (and (eq value 'transparent)
                     (not (exwm-systemtray--transparency-supported-p)))
            (display-warning 'exwm-systemtray
                             "Transparent background is not supported yet when \
-using 32-bit depth.  Using `workspace-background' instead.")
-           (setq value 'workspace-background))
+using 32-bit depth.  Using `default' instead.")
+           (setq value 'default))
          (set-default symbol value)
          (when (and exwm-systemtray-mode
                     exwm-systemtray--connection
@@ -113,6 +119,11 @@ using 32-bit depth.  Using `workspace-background' instead.")
                                         :window exwm-systemtray--embedder-window))
            (xcb:flush exwm-systemtray--connection))))
 
+(defvar exwm-systemtray-width 0
+  "Current width of systemtray, which is computed when the icons are refreshed.
+The width is useful to adjust the tab-bar alignment when
+`exwm-systemtray-position' is set to `top'.")
+
 ;; GTK icons require at least 16 pixels to show normally.
 (defconst exwm-systemtray--icon-min-size 16 "Minimum icon size.")
 
@@ -253,6 +264,7 @@ using 32-bit depth.  Using `workspace-background' instead.")
                                              xcb:ConfigWindow:Width)
                          :x (- (slot-value workarea 'width) x)
                          :width x)))
+    (setq exwm-systemtray-width x)
     (when map
       (xcb:+request exwm-systemtray--connection
           (make-instance 'xcb:MapWindow
@@ -263,8 +275,7 @@ using 32-bit depth.  Using `workspace-background' instead.")
   "Refresh background color after theme change or workspace switch.
 If REMAP is not nil, map and unmap the embedder window so that the background is
 redrawn."
-  ;; Only `workspace-background' is dependent on current theme and workspace.
-  (when (eq 'workspace-background exwm-systemtray-background-color)
+  (when (facep exwm-systemtray-background-color)
     (exwm-systemtray--set-background-color)
     (when remap
       (xcb:+request exwm-systemtray--connection
@@ -283,16 +294,17 @@ Note that this function does not change the current contents of the embedder
 window; unmap & map are necessary for the background color to take effect."
   (when (and exwm-systemtray--connection
              exwm-systemtray--embedder-window)
-    (let* ((color (cl-case exwm-systemtray-background-color
-                    ((transparent nil) ; nil means transparent as well
+    (let* ((color (pcase exwm-systemtray-background-color
+                    ((or `transparent `nil) ; nil means transparent as well
                      (if (exwm-systemtray--transparency-supported-p)
                          nil
                        (message "%s" "[EXWM] system tray does not support \
-`transparent' background; using `workspace-background' instead")
+`transparent' background; using `default' instead")
                        (face-background 'default exwm-workspace--current)))
-                    (workspace-background
-                     (face-background 'default exwm-workspace--current))
-                    (t exwm-systemtray-background-color)))
+                    ((pred facep)
+                     (face-background exwm-systemtray-background-color
+                                      exwm-workspace--current))
+                    (_ exwm-systemtray-background-color)))
            (background-pixel (exwm--color->pixel color)))
       (xcb:+request exwm-systemtray--connection
                     (make-instance 'xcb:ChangeWindowAttributes
@@ -448,6 +460,20 @@ Argument DATA contains the raw event data."
                        :event (xcb:marshal obj exwm-systemtray--connection))))
   (xcb:flush exwm-systemtray--connection))
 
+(defun exwm-systemtray--y-position ()
+  "Y position of system tray."
+  (let ((pos exwm-systemtray-position))
+    (cond
+     ((or (eq pos 'bottom) (and (fixnump pos) (<= pos 0)))
+      (- (slot-value (exwm-workspace--workarea
+                       exwm-workspace-current-index)
+                     'height)
+         exwm-workspace--frame-y-offset
+         exwm-systemtray-height
+         (if (fixnump pos) (- pos) 0)))
+     ((fixnump pos) pos)
+     (t 0))))
+
 (defun exwm-systemtray--on-workspace-switch ()
   "Reparent/Refresh the system tray in `exwm-workspace-switch-hook'."
   (exwm--log)
@@ -460,11 +486,7 @@ Argument DATA contains the raw event data."
                                 (frame-parameter exwm-workspace--current
                                                  'window-id))
                        :x 0
-                       :y (- (slot-value (exwm-workspace--workarea
-                                           exwm-workspace-current-index)
-                                         'height)
-                             exwm-workspace--frame-y-offset
-                             exwm-systemtray-height))))
+                       :y (exwm-systemtray--y-position))))
   (exwm-systemtray--refresh-background-color)
   (exwm-systemtray--refresh))
 
@@ -481,11 +503,7 @@ Argument DATA contains the raw event data."
         (make-instance 'xcb:ConfigureWindow
                        :window exwm-systemtray--embedder-window
                        :value-mask xcb:ConfigWindow:Y
-                       :y (- (slot-value (exwm-workspace--workarea
-                                           exwm-workspace-current-index)
-                                         'height)
-                             exwm-workspace--frame-y-offset
-                             exwm-systemtray-height))))
+                       :y (exwm-systemtray--y-position))))
   (exwm-systemtray--refresh))
 
 (cl-defun exwm-systemtray--init ()
@@ -579,11 +597,7 @@ Argument DATA contains the raw event data."
       (exwm-workspace--update-offsets)
       (setq frame exwm-workspace--current
             ;; Bottom aligned.
-            y (- (slot-value (exwm-workspace--workarea
-                               exwm-workspace-current-index)
-                             'height)
-                 exwm-workspace--frame-y-offset
-                 exwm-systemtray-height)))
+            y (exwm-systemtray--y-position)))
     (setq parent (string-to-number (frame-parameter frame 'window-id)))
     ;; Use default depth, visual and colormap (from root window), instead of
     ;; Emacs frame's.  See Section "Visual and background pixmap handling" in
@@ -694,8 +708,5 @@ Argument DATA contains the raw event data."
     (when (boundp 'exwm-randr-refresh-hook)
       (remove-hook 'exwm-randr-refresh-hook #'exwm-systemtray--refresh-all))))
 
-
-
 (provide 'exwm-systemtray)
-
 ;;; exwm-systemtray.el ends here
